/*
 * The MIT License
 *
 * Copyright 2017 Germán Bordel <german.bordel at ehu.es>.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.upvehu.gb.paralelismo;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

/**
 * JFrame con un ejemplo concreto de ejecución de tareas en paralelo.
 * 
 * @author Germán Bordel <german.bordel at ehu.es>
 */
public class ParalelismoGUI extends javax.swing.JFrame {
    private String txt;
    
    /**
     * Creates new form ParalelismoGUI
     * @param txt el texto a mostrar en el panel de "texto lento".
     */
    public ParalelismoGUI(String txt) {
        initComponents();
       
        //Asunto "texto lento"
        this.txt=txt;
        textDelayControl(null);
        
        //Asunto Panel con partícula
        Thread t=new Thread(panelBrowniano);t.setDaemon(true);t.start();
        
        //Reloj
        new Timer().schedule(new TimerTask(){
                    @Override
                    public void run() {
                        DateFormat df=new SimpleDateFormat("hh:mm:ss");
                        reloj.setText(df.format(new Date()));
                     }
                }, 0, 1000);
    }

    /**
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        textDelay = new javax.swing.JSpinner();
        jScrollPane1 = new javax.swing.JScrollPane();
        timedText = new javax.swing.JTextPane();
        panelBrowniano = new edu.upvehu.gb.paralelismo.JPanelBrowniano();
        jPanel3 = new javax.swing.JPanel();
        reloj = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setMaximumSize(new java.awt.Dimension(800, 600));
        setPreferredSize(new java.awt.Dimension(800, 600));

        jPanel1.setLayout(new java.awt.GridLayout(1, 2));

        jPanel2.setLayout(new java.awt.BorderLayout());

        jLabel2.setText("Décimas de segundo [1..10]:");
        jPanel4.add(jLabel2);

        textDelay.setModel(new javax.swing.SpinnerNumberModel(5, 1, 10, 1));
        textDelay.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                textDelayControl(evt);
            }
        });
        jPanel4.add(textDelay);

        jPanel2.add(jPanel4, java.awt.BorderLayout.PAGE_START);

        timedText.setText("En este área de texto va apareciendo despacio, letra a letra, un texto predefinido que puede ser cualquiera que se elija. Cuando se termine de mostrar se borrará y  volverá a acomenzar.\n\nEn esta aplicación de ejemplo el texto es esta explicación, pero podría ser EL Quijote o cualquier otra cosa. \n\nEn un lugar de la Mancha de cuyo nom");
        jScrollPane1.setViewportView(timedText);

        jPanel2.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel1.add(jPanel2);

        panelBrowniano.setBackground(new java.awt.Color(0, 102, 102));
        panelBrowniano.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        javax.swing.GroupLayout panelBrownianoLayout = new javax.swing.GroupLayout(panelBrowniano);
        panelBrowniano.setLayout(panelBrownianoLayout);
        panelBrownianoLayout.setHorizontalGroup(
            panelBrownianoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 223, Short.MAX_VALUE)
        );
        panelBrownianoLayout.setVerticalGroup(
            panelBrownianoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 387, Short.MAX_VALUE)
        );

        jPanel1.add(panelBrowniano);

        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        reloj.setFont(new java.awt.Font("Dialog", 1, 24)); // NOI18N
        reloj.setForeground(new java.awt.Color(51, 204, 0));
        reloj.setText("00:00:00");
        jPanel3.add(reloj);

        getContentPane().add(jPanel3, java.awt.BorderLayout.PAGE_START);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private int pos=0;
    private Timer textTimer=new Timer();
    private void textDelayControl(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_textDelayControl
        textTimer.cancel();
        textTimer=new Timer(true);
        textTimer.schedule(new TimerTask(){
                    @Override
                    public void run() {
                        if (pos==txt.length()) pos=0;
                        timedText.setText(txt.substring(0,++pos));
                   }
                }, 0, (int)textDelay.getValue()*100);
    }//GEN-LAST:event_textDelayControl

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private edu.upvehu.gb.paralelismo.JPanelBrowniano panelBrowniano;
    private javax.swing.JLabel reloj;
    private javax.swing.JSpinner textDelay;
    private javax.swing.JTextPane timedText;
    // End of variables declaration//GEN-END:variables
}
